@@ -28,7 +28,7 @@
 
 #include <compiz-plugin.h>
 
-#define CORE_ABIVERSION 20071103
+#define CORE_ABIVERSION 20080214
 
 #include <stdio.h>
 #include <sys/time.h>
@@ -344,6 +344,43 @@ compObjectFind (CompObject     *parent,
     else						     \
 	return (def)
 
+/* session.c */
+
+typedef void (*SessionSaveYourselfProc) (CompCore   *c,
+					 const char *clientId,
+					 int        saveType,
+					 int        interactStyle,
+					 Bool       shutdown,
+					 Bool       fast);
+
+typedef void (*SessionDieProc) (CompCore *c);
+
+typedef void (*SessionSaveCompleteProc) (CompCore *c);
+
+typedef void (*SessionShutdownCancelledProc) (CompCore *c);
+
+void
+initSession (char *smPrevClientId);
+
+void
+closeSession (void);
+
+void
+sessionSaveYourself (CompCore   *c,
+		     const char *clientId,
+		     int        saveType,
+		     int        interactStyle,
+		     Bool       shutdown,
+		     Bool       fast);
+
+void
+sessionDie (CompCore *c);
+
+void
+sessionSaveComplete (CompCore *c);
+
+void
+sessionShutdownCancelled (CompCore *c);
 
 /* option.c */
 
@@ -647,6 +684,11 @@ struct _CompCore {
 
     FileWatchAddedProc   fileWatchAdded;
     FileWatchRemovedProc fileWatchRemoved;
+
+    SessionSaveYourselfProc      sessionSaveYourself;
+    SessionDieProc               sessionDie;
+    SessionSaveCompleteProc      sessionSaveComplete;
+    SessionShutdownCancelledProc sessionShutdownCancelled;
 };
 
 int
@@ -3088,15 +3130,6 @@ getPluginDisplayIndex (CompDisplay *d,
 		       int	   *index);
 
 
-/* session.c */
-
-void
-initSession (char *smPrevClientId);
-
-void
-closeSession (void);
-
-
 /* fragment.c */
 
 #define MAX_FRAGMENT_FUNCTIONS 16
