@@ -259,6 +259,7 @@ annoDrawLine (CompScreen     *s,
 	cairo_move_to (cr, x1, y1);
 	cairo_line_to (cr, x2, y2);
 	cairo_stroke_extents (cr, &ex1, &ey1, &ex2, &ey2);
+	cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
 	annoSetSourceColor (cr, color);
 	cairo_stroke (cr);
 
@@ -742,7 +743,7 @@ annoSetDisplayOption (CompPlugin      *plugin,
 static const CompMetadataOptionInfo annoDisplayOptionInfo[] = {
     { "initiate_button", "button", 0, annoInitiate, annoTerminate },
     { "draw_button", "button", 0, annoDraw, 0 },
-    { "erase_button", "button", 0, annoEraseInitiate, 0 },
+    { "erase_button", "button", 0, annoEraseInitiate, annoTerminate },
     { "clear_key", "key", 0, annoClear, 0 },
     { "clear_button", "button", 0, annoClear, 0 },
     { "fill_color", "color", 0, 0, 0 },
