@@ -27,7 +27,7 @@
 #include <string.h>
 #include <math.h>
 
-#include <compiz.h>
+#include <compiz-core.h>
 #include <decoration.h>
 
 #include <X11/Xatom.h>
@@ -68,6 +68,12 @@ typedef struct _BlurState {
     Bool    clipped;
 } BlurState;
 
+static int corePrivateIndex;
+
+typedef struct _BlurCore {
+    ObjectAddProc objectAdd;
+} BlurCore;
+
 static int displayPrivateIndex;
 
 #define BLUR_DISPLAY_OPTION_PULSE 0
@@ -110,7 +116,6 @@ typedef struct _BlurScreen {
     DrawWindowProc	         drawWindow;
     DrawWindowTextureProc        drawWindowTexture;
 
-    WindowAddNotifyProc    windowAddNotify;
     WindowResizeNotifyProc windowResizeNotify;
     WindowMoveNotifyProc   windowMoveNotify;
 
@@ -167,20 +172,26 @@ typedef struct _BlurWindow {
     Region clip;
 } BlurWindow;
 
-#define GET_BLUR_DISPLAY(d)				     \
-    ((BlurDisplay *) (d)->privates[displayPrivateIndex].ptr)
+#define GET_BLUR_CORE(c)				    \
+    ((BlurCore *) (c)->base.privates[corePrivateIndex].ptr)
+
+#define BLUR_CORE(c)		     \
+    BlurCore *bc = GET_BLUR_CORE (c)
+
+#define GET_BLUR_DISPLAY(d)					  \
+    ((BlurDisplay *) (d)->base.privates[displayPrivateIndex].ptr)
 
 #define BLUR_DISPLAY(d)			   \
     BlurDisplay *bd = GET_BLUR_DISPLAY (d)
 
-#define GET_BLUR_SCREEN(s, bd)					 \
-    ((BlurScreen *) (s)->privates[(bd)->screenPrivateIndex].ptr)
+#define GET_BLUR_SCREEN(s, bd)					      \
+    ((BlurScreen *) (s)->base.privates[(bd)->screenPrivateIndex].ptr)
 
 #define BLUR_SCREEN(s)							\
     BlurScreen *bs = GET_BLUR_SCREEN (s, GET_BLUR_DISPLAY (s->display))
 
-#define GET_BLUR_WINDOW(w, bs)					 \
-    ((BlurWindow *) (w)->privates[(bs)->windowPrivateIndex].ptr)
+#define GET_BLUR_WINDOW(w, bs)					      \
+    ((BlurWindow *) (w)->base.privates[(bs)->windowPrivateIndex].ptr)
 
 #define BLUR_WINDOW(w)					     \
     BlurWindow *bw = GET_BLUR_WINDOW  (w,		     \
@@ -2399,9 +2410,10 @@ blurMatchPropertyChanged (CompDisplay *d,
 }
 
 static void
-blurWindowAdd (CompWindow *w)
+blurWindowAdd (CompScreen *s,
+	       CompWindow *w)
 {
-    BLUR_SCREEN (w->screen);
+    BLUR_SCREEN (s);
 
     blurWindowUpdate (w, BLUR_STATE_CLIENT);
     blurWindowUpdate (w, BLUR_STATE_DECOR);
@@ -2410,15 +2422,63 @@ blurWindowAdd (CompWindow *w)
 }
 
 static void
-blurWindowAddNotify (CompWindow *w)
+blurObjectAdd (CompObject *parent,
+	       CompObject *object)
 {
-    BLUR_SCREEN (w->screen);
+    static ObjectAddProc dispTab[] = {
+	(ObjectAddProc) 0, /* CoreAdd */
+	(ObjectAddProc) 0, /* DisplayAdd */
+	(ObjectAddProc) 0, /* ScreenAdd */
+	(ObjectAddProc) blurWindowAdd
+    };
 
-    blurWindowAdd (w);
+    BLUR_CORE (&core);
 
-    UNWRAP (bs, w->screen, windowAddNotify);
-    (*w->screen->windowAddNotify) (w);
-    WRAP (bs, w->screen, windowAddNotify, blurWindowAddNotify);
+    UNWRAP (bc, &core, objectAdd);
+    (*core.objectAdd) (parent, object);
+    WRAP (bc, &core, objectAdd, blurObjectAdd);
+
+    DISPATCH (object, dispTab, ARRAY_SIZE (dispTab), (parent, object));
+}
+
+static Bool
+blurInitCore (CompPlugin *p,
+	      CompCore   *c)
+{
+    BlurCore *bc;
+
+    if (!checkPluginABI ("core", CORE_ABIVERSION))
+	return FALSE;
+
+    bc = malloc (sizeof (BlurCore));
+    if (!bc)
+	return FALSE;
+
+    displayPrivateIndex = allocateDisplayPrivateIndex ();
+    if (displayPrivateIndex < 0)
+    {
+	free (bc);
+	return FALSE;
+    }
+
+    WRAP (bc, c, objectAdd, blurObjectAdd);
+
+    c->base.privates[corePrivateIndex].ptr = bc;
+
+    return TRUE;
+}
+
+static void
+blurFiniCore (CompPlugin *p,
+	      CompCore   *c)
+{
+    BLUR_CORE (c);
+
+    freeDisplayPrivateIndex (displayPrivateIndex);
+
+    UNWRAP (bc, c, objectAdd);
+
+    free (bc);
 }
 
 static const CompMetadataOptionInfo blurDisplayOptionInfo[] = {
@@ -2456,13 +2516,13 @@ blurInitDisplay (CompPlugin  *p,
     bd->blurAtom[BLUR_STATE_CLIENT] =
 	XInternAtom (d->display, "_COMPIZ_WM_WINDOW_BLUR", 0);
     bd->blurAtom[BLUR_STATE_DECOR] =
-	XInternAtom (d->display, "_COMPIZ_WM_WINDOW_BLUR_DECOR", 0);
+	XInternAtom (d->display, DECOR_BLUR_ATOM_NAME, 0);
 
     WRAP (bd, d, handleEvent, blurHandleEvent);
     WRAP (bd, d, matchExpHandlerChanged, blurMatchExpHandlerChanged);
     WRAP (bd, d, matchPropertyChanged, blurMatchPropertyChanged);
 
-    d->privates[displayPrivateIndex].ptr = bd;
+    d->base.privates[displayPrivateIndex].ptr = bd;
 
     return TRUE;
 }
@@ -2623,11 +2683,10 @@ blurInitScreen (CompPlugin *p,
     WRAP (bs, s, paintWindow, blurPaintWindow);
     WRAP (bs, s, drawWindow, blurDrawWindow);
     WRAP (bs, s, drawWindowTexture, blurDrawWindowTexture);
-    WRAP (bs, s, windowAddNotify, blurWindowAddNotify);
     WRAP (bs, s, windowResizeNotify, blurWindowResizeNotify);
     WRAP (bs, s, windowMoveNotify, blurWindowMoveNotify);
 
-    s->privates[bd->screenPrivateIndex].ptr = bs;
+    s->base.privates[bd->screenPrivateIndex].ptr = bs;
 
     blurUpdateFilterRadius (s);
 
@@ -2669,7 +2728,6 @@ blurFiniScreen (CompPlugin *p,
     UNWRAP (bs, s, paintWindow);
     UNWRAP (bs, s, drawWindow);
     UNWRAP (bs, s, drawWindowTexture);
-    UNWRAP (bs, s, windowAddNotify);
     UNWRAP (bs, s, windowResizeNotify);
     UNWRAP (bs, s, windowMoveNotify);
 
@@ -2715,10 +2773,10 @@ blurInitWindow (CompPlugin *p,
 	return FALSE;
     }
 
-    w->privates[bs->windowPrivateIndex].ptr = bw;
+    w->base.privates[bs->windowPrivateIndex].ptr = bw;
 
-    if (w->added)
-	blurWindowAdd (w);
+    if (w->base.parent)
+	blurWindowAdd (w->screen, w);
 
     return TRUE;
 }
@@ -2743,6 +2801,65 @@ blurFiniWindow (CompPlugin *p,
     free (bw);
 }
 
+static CompBool
+blurInitObject (CompPlugin *p,
+		CompObject *o)
+{
+    static InitPluginObjectProc dispTab[] = {
+	(InitPluginObjectProc) blurInitCore,
+	(InitPluginObjectProc) blurInitDisplay,
+	(InitPluginObjectProc) blurInitScreen,
+	(InitPluginObjectProc) blurInitWindow
+    };
+
+    RETURN_DISPATCH (o, dispTab, ARRAY_SIZE (dispTab), TRUE, (p, o));
+}
+
+static void
+blurFiniObject (CompPlugin *p,
+		CompObject *o)
+{
+    static FiniPluginObjectProc dispTab[] = {
+	(FiniPluginObjectProc) blurFiniCore,
+	(FiniPluginObjectProc) blurFiniDisplay,
+	(FiniPluginObjectProc) blurFiniScreen,
+	(FiniPluginObjectProc) blurFiniWindow
+    };
+
+    DISPATCH (o, dispTab, ARRAY_SIZE (dispTab), (p, o));
+}
+
+static CompOption *
+blurGetObjectOptions (CompPlugin *plugin,
+		      CompObject *object,
+		      int	 *count)
+{
+    static GetPluginObjectOptionsProc dispTab[] = {
+	(GetPluginObjectOptionsProc) 0, /* GetCoreOptions */
+	(GetPluginObjectOptionsProc) blurGetDisplayOptions,
+	(GetPluginObjectOptionsProc) blurGetScreenOptions
+    };
+
+    RETURN_DISPATCH (object, dispTab, ARRAY_SIZE (dispTab),
+		     (void *) (*count = 0), (plugin, object, count));
+}
+
+static CompBool
+blurSetObjectOption (CompPlugin      *plugin,
+		     CompObject      *object,
+		     const char      *name,
+		     CompOptionValue *value)
+{
+    static SetPluginObjectOptionProc dispTab[] = {
+	(SetPluginObjectOptionProc) 0, /* SetCoreOption */
+	(SetPluginObjectOptionProc) blurSetDisplayOption,
+	(SetPluginObjectOptionProc) blurSetScreenOption
+    };
+
+    RETURN_DISPATCH (object, dispTab, ARRAY_SIZE (dispTab), FALSE,
+		     (plugin, object, name, value));
+}
+
 static Bool
 blurInit (CompPlugin *p)
 {
@@ -2754,8 +2871,8 @@ blurInit (CompPlugin *p)
 					 BLUR_SCREEN_OPTION_NUM))
 	return FALSE;
 
-    displayPrivateIndex = allocateDisplayPrivateIndex ();
-    if (displayPrivateIndex < 0)
+    corePrivateIndex = allocateCorePrivateIndex ();
+    if (corePrivateIndex < 0)
     {
 	compFiniMetadata (&blurMetadata);
 	return FALSE;
@@ -2769,17 +2886,10 @@ blurInit (CompPlugin *p)
 static void
 blurFini (CompPlugin *p)
 {
-    freeDisplayPrivateIndex (displayPrivateIndex);
+    freeCorePrivateIndex (corePrivateIndex);
     compFiniMetadata (&blurMetadata);
 }
 
-static int
-blurGetVersion (CompPlugin *plugin,
-		int	   version)
-{
-    return ABIVERSION;
-}
-
 static CompMetadata *
 blurGetMetadata (CompPlugin *plugin)
 {
@@ -2788,24 +2898,17 @@ blurGetMetadata (CompPlugin *plugin)
 
 static CompPluginVTable blurVTable = {
     "blur",
-    blurGetVersion,
     blurGetMetadata,
     blurInit,
     blurFini,
-    blurInitDisplay,
-    blurFiniDisplay,
-    blurInitScreen,
-    blurFiniScreen,
-    blurInitWindow,
-    blurFiniWindow,
-    blurGetDisplayOptions,
-    blurSetDisplayOption,
-    blurGetScreenOptions,
-    blurSetScreenOption
+    blurInitObject,
+    blurFiniObject,
+    blurGetObjectOptions,
+    blurSetObjectOption
 };
 
 CompPluginVTable *
-getCompPluginInfo (void)
+getCompPluginInfo20070830 (void)
 {
     return &blurVTable;
 }
