@@ -1086,7 +1086,16 @@ scaleTerminate (CompDisplay     *d,
 		    }
 		}
 
-		if (ss->state != SCALE_STATE_IN)
+		if (state & CompActionStateCancel)
+		{
+		    if (d->activeWindow != sd->previousActiveWindow)
+		    {
+			w = findWindowAtScreen (s, sd->previousActiveWindow);
+			if (w)
+			    moveInputFocusToWindow (w);
+		    }
+		}
+		else if (ss->state != SCALE_STATE_IN)
 		{
 		    w = findWindowAtScreen (s, sd->lastActiveWindow);
 		    if (w)
@@ -1113,6 +1122,8 @@ scaleTerminate (CompDisplay     *d,
 	}
     }
 
+    action->state &= ~(CompActionStateTermKey | CompActionStateTermButton);
+
     return FALSE;
 }
 
@@ -1198,9 +1209,10 @@ scaleInitiateCommon (CompScreen      *s,
 	if (!sd->lastActiveNum)
 	    sd->lastActiveNum = s->activeNum - 1;
 
-	sd->lastActiveWindow = s->display->activeWindow;
-	sd->selectedWindow   = s->display->activeWindow;
-	sd->hoveredWindow    = None;
+	sd->previousActiveWindow = s->display->activeWindow;
+	sd->lastActiveWindow     = s->display->activeWindow;
+	sd->selectedWindow       = s->display->activeWindow;
+	sd->hoveredWindow        = None;
 
 	ss->state = SCALE_STATE_OUT;
 
@@ -1243,7 +1255,8 @@ scaleInitiate (CompDisplay     *d,
 	else if ((state & EDGE_STATE) && ss->state == SCALE_STATE_WAIT)
 	{
 	    if (ss->type == ScaleTypeNormal)
-		return scaleTerminate (s->display, action, 0, option, nOption);
+		return scaleTerminate (s->display, action,
+				       CompActionStateCancel, option, nOption);
 	}
     }
 
@@ -1275,7 +1288,8 @@ scaleInitiateAll (CompDisplay     *d,
 	else if ((state & EDGE_STATE) && ss->state == SCALE_STATE_WAIT)
 	{
 	    if (ss->type == ScaleTypeAll)
-		return scaleTerminate (s->display, action, 0, option, nOption);
+		return scaleTerminate (s->display, action,
+				       CompActionStateCancel, option, nOption);
 	}
     }
 
@@ -1316,7 +1330,8 @@ scaleInitiateGroup (CompDisplay     *d,
 	else if ((state & EDGE_STATE) && ss->state == SCALE_STATE_WAIT)
 	{
 	    if (ss->type == ScaleTypeGroup)
-		return scaleTerminate (s->display, action, 0, option, nOption);
+		return scaleTerminate (s->display, action,
+				       CompActionStateCancel, option, nOption);
 	}
     }
 
@@ -1348,45 +1363,14 @@ scaleInitiateOutput (CompDisplay     *d,
 	else if ((state & EDGE_STATE) && ss->state == SCALE_STATE_WAIT)
 	{
 	    if (ss->type == ScaleTypeOutput)
-		return scaleTerminate (s->display, action, 0, option, nOption);
+		return scaleTerminate (s->display, action,
+				       CompActionStateCancel, option, nOption);
 	}
     }
 
     return FALSE;
 }
 
-static Bool
-scaleRelayoutSlots (CompDisplay     *d,
-		    CompAction      *action,
-		    CompActionState state,
-		    CompOption      *option,
-		    int	            nOption)
-{
-    CompScreen *s;
-    Window     xid;
-
-    xid = getIntOptionNamed (option, nOption, "root", 0);
-
-    s = findScreenAtDisplay (d, xid);
-    if (s)
-    {
-	SCALE_SCREEN (s);
-
-	if (ss->state != SCALE_STATE_NONE && ss->state != SCALE_STATE_IN)
-	{
-	    if (layoutThumbs (s))
-	    {
-		ss->state = SCALE_STATE_OUT;
-		damageScreen (s);
-	    }
-	}
-
-	return TRUE;
-    }
-
-    return FALSE;
-}
-
 static void
 scaleSelectWindow (CompWindow *w)
 
@@ -1424,7 +1408,9 @@ scaleSelectWindowAt (CompScreen *s,
     w = scaleCheckForWindowAt (s, x, y);
     if (w && isScaleWin (w))
     {
-	scaleSelectWindow (w);
+	SCALE_SCREEN (s);
+
+	(*ss->selectWindow) (w);
 
 	if (moveInputFocus)
 	{
@@ -1515,8 +1501,9 @@ scaleMoveFocusWindow (CompScreen *s,
     if (focus)
     {
 	SCALE_DISPLAY (s->display);
+	SCALE_SCREEN (s);
 
-	scaleSelectWindow (focus);
+	(*ss->selectWindow) (focus);
 
 	sd->lastActiveNum    = focus->activeNum;
 	sd->lastActiveWindow = focus->id;
@@ -1525,6 +1512,39 @@ scaleMoveFocusWindow (CompScreen *s,
     }
 }
 
+static Bool
+scaleRelayoutSlots (CompDisplay     *d,
+		    CompAction      *action,
+		    CompActionState state,
+		    CompOption      *option,
+		    int	            nOption)
+{
+    CompScreen *s;
+    Window     xid;
+
+    xid = getIntOptionNamed (option, nOption, "root", 0);
+
+    s = findScreenAtDisplay (d, xid);
+    if (s)
+    {
+	SCALE_SCREEN (s);
+
+	if (ss->state != SCALE_STATE_NONE && ss->state != SCALE_STATE_IN)
+	{
+	    if (layoutThumbs (s))
+	    {
+		ss->state = SCALE_STATE_OUT;
+		scaleMoveFocusWindow (s, 0, 0);
+		damageScreen (s);
+	    }
+	}
+
+	return TRUE;
+    }
+
+    return FALSE;
+}
+
 static void
 scaleWindowRemove (CompDisplay *d,
 		   Window      id)
@@ -1567,11 +1587,13 @@ scaleWindowRemove (CompDisplay *d,
 
 			opt = SCALE_DISPLAY_OPTION_INITIATE_EDGE;
 			action = &sd->opt[opt].value.action;
-			scaleTerminate (d, action, 0, &o, 1);
+			scaleTerminate (d, action,
+					CompActionStateCancel, &o, 1);
 
 			opt = SCALE_DISPLAY_OPTION_INITIATE_KEY;
 			action = &sd->opt[opt].value.action;
-			scaleTerminate (d, action, 0, &o, 1);
+			scaleTerminate (d, action,
+					CompActionStateCancel, &o, 1);
 			break;
 		    }
 		}
@@ -2063,6 +2085,7 @@ scaleInitScreen (CompPlugin *p,
     ss->layoutSlotsAndAssignWindows = layoutSlotsAndAssignWindows;
     ss->setScaledPaintAttributes    = setScaledPaintAttributes;
     ss->scalePaintDecoration	    = scalePaintDecoration;
+    ss->selectWindow                = scaleSelectWindow;
 
     WRAP (ss, s, preparePaintScreen, scalePreparePaintScreen);
     WRAP (ss, s, donePaintScreen, scaleDonePaintScreen);
