@@ -1381,7 +1381,7 @@ enterShowDesktopMode (CompScreen *s)
 	if ((s->showingDesktopMask & w->wmType) &&
 	    (!(w->state & CompWindowStateSkipTaskbarMask) || st->value.b))
 	{
-	    if (!w->inShowDesktopMode && (*s->focusWindow) (w))
+	    if (!w->inShowDesktopMode && !w->grabbed && (*s->focusWindow) (w))
 	    {
 		w->inShowDesktopMode = TRUE;
 		hideWindow (w);
@@ -3128,9 +3128,16 @@ updateWorkareaForScreen (CompScreen *s)
 
     if (memcmp (&workArea, &s->workArea, sizeof (XRectangle)))
     {
+	CompWindow *w;
+
 	s->workArea = workArea;
 
 	setDesktopHints (s);
+
+	/* as work area changed, update all maximized windows on this
+	   screen to snap to the new work area */
+	for (w = s->windows; w; w = w->next)
+	    updateWindowSize (w);
     }
 }
 
