@@ -82,10 +82,36 @@ setStringListProperty (SmcConn	  connection,
 static void
 setCloneRestartCommands (SmcConn connection)
 {
-    setStringListProperty (connection, SmCloneCommand,
-			   (const char **) programArgv, programArgc);
-    setStringListProperty (connection, SmRestartCommand,
-			   (const char **) programArgv, programArgc);
+    const char **args;
+    int        i, count = 0;
+
+    /* at maximum, we pass our old arguments + our new client id
+       to the SM, so allocate for that case */
+    args = malloc ((programArgc + 2) * sizeof (char *));
+    if (!args)
+	return;
+
+    for (i = 0; i < programArgc; i++)
+    {
+	if (strcmp (programArgv[i], "--sm-client-id") == 0)
+	    i++; /* skip old client id, we'll add the new one later */
+	else
+	    args[count++] = programArgv[i];
+    }
+
+    setStringListProperty (connection, SmCloneCommand, args, count);
+
+    /* insert new client id at position 1 and 2;
+       position 0 is the executable name */
+    for (i = count - 1; i >= 1; i--)
+	args[i + 2] = args[i];
+    args[1] = "--sm-client-id";
+    args[2] = smClientId;
+    count += 2;
+
+    setStringListProperty (connection, SmRestartCommand, args, count);
+
+    free (args);
 }
 
 static void
@@ -106,44 +132,6 @@ setRestartStyle (SmcConn connection, char hint)
     SmcSetProperties (connection, 1, &pProp);
 }
 
-static void
-saveYourselfGotProps (SmcConn   connection,
-		      SmPointer client_data,
-		      int       num_props,
-		      SmProp    **props)
-{
-    int p, i;
-
-    for (p = 0; p < num_props; p++)
-    {
-	if (!strcmp (props[p]->name, SmRestartCommand))
-	{
-	    for (i = 0; i < props[p]->num_vals - 1; i++)
-	    {
-		if (!strncmp (props[p]->vals[i].value,
-			      "--sm-client-id",
-			      props[p]->vals[i].length))
-		{
-		    SmPropValue oldVal = props[p]->vals[i + 1];
-
-		    props[p]->vals[i + 1].value = smClientId;
-		    props[p]->vals[i + 1].length = strlen (smClientId);
-		    SmcSetProperties (connection, 1, &props[p]);
-		    props[p]->vals[i + 1] = oldVal;
-
-		    goto out;
-		}
-	    }
-	}
-    }
-
-out:
-    setRestartStyle (connection, SmRestartImmediately);
-    setCloneRestartCommands (connection);
-
-    SmcSaveYourselfDone (connection, 1);
-}
-
 static void
 saveYourselfCallback (SmcConn	connection,
 		      SmPointer client_data,
@@ -152,14 +140,21 @@ saveYourselfCallback (SmcConn	connection,
 		      int	interact_Style,
 		      Bool	fast)
 {
-    if (!SmcGetProperties (connection, saveYourselfGotProps, NULL))
-	SmcSaveYourselfDone (connection, 1);
+    (*core.sessionSaveYourself) (&core, smClientId,
+				 saveType, interact_Style,
+				 shutdown, fast);
+
+    setCloneRestartCommands (connection);
+    setRestartStyle (connection, SmRestartImmediately);
+    SmcSaveYourselfDone (connection, 1);
 }
 
 static void
 dieCallback (SmcConn   connection,
 	     SmPointer clientData)
 {
+    (*core.sessionDie) (&core);
+
     closeSession ();
     exit (0);
 }
@@ -168,12 +163,14 @@ static void
 saveCompleteCallback (SmcConn	connection,
 		      SmPointer clientData)
 {
+    (*core.sessionSaveComplete) (&core);
 }
 
 static void
 shutdownCancelledCallback (SmcConn   connection,
 			   SmPointer clientData)
 {
+    (*core.sessionShutdownCancelled) (&core);
 }
 
 void
@@ -237,6 +234,31 @@ closeSession (void)
     }
 }
 
+void
+sessionSaveYourself (CompCore   *c,
+		     const char *clientId,
+		     int        saveType,
+		     int        interactStyle,
+		     Bool       shutdown,
+		     Bool       fast)
+{
+}
+
+void
+sessionDie (CompCore *c)
+{
+}
+
+void
+sessionSaveComplete (CompCore *c)
+{
+}
+
+void
+sessionShutdownCancelled (CompCore *c)
+{
+}
+
 /* ice connection handling taken and updated from gnome-ice.c
  * original gnome-ice.c code written by Tom Tromey <tromey@cygnus.com>
  */
